/*
   FNT - Daemon for handle xmpp-connections
   Reading application settings

   Copyright (C) 2009 Free Software Foundation, Inc.

   Written by:
   Slava Zanko <slavazanko@gmail.com>


   This file is part of the FIDO New Technology.

   FIDO New Technology is free software: you can redistribute it and/or 
   modify it under the terms of the GNU General Public License as published 
   by the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#	include <config.h>
#endif

#include <gnutls/gnutls.h>
#include <gnutls/x509.h>

#include "../include/global.h"
#include "../include/types.h"
#include "../include/sql.h"
#include "../include/sql-config.h"
#include "../include/log.h"
#include "../include/ssltls.h"

#include "settings.h"
#include "main.h"

/*** global variables **************************************************/

/*** file scope macro definitions **************************************/

#define CONFIG_GROUP PACKAGE"-kernel"

/*** file scope type declarations **************************************/

/*** file scope variables **********************************************/

/*** file scope functions **********************************************/

/*** public functions **************************************************/

void
kernel_settings_cleanup (void)
{
    if (Application.settings.isSettingsInit == false)
	return;
    kernel_settings_cleanup ();

    g_free (Application.settings.listen_ip);
    g_free (Application.settings.pid_file);
    g_free (Application.settings.pid_directory);
    g_free (Application.settings.ssl_directory);
    g_free (Application.settings.ssl_private_key);
    g_free (Application.settings.ssl_public_key);
    g_free (Application.settings.fidonet_node_address);

    if (Application.settings.facility_or_fname != NULL)
	g_free (Application.settings.facility_or_fname);
}

bool
kernel_settings_read (void)
{
    char *tmp_param;
    kernel_settings_cleanup ();

    Application.settings.pid_directory =
	fnt_sql_config_getParameter_str ("common-settings", "pid.directory",
					 ".");

    Application.settings.ssl_directory =
	fnt_sql_config_getParameter_str ("common-settings", "ssl.directory",
					 ".");

    Application.settings.fidonet_node_address =
	fnt_sql_config_getParameter_str ("fidonet", "node.address", "");

    tmp_param =
	fnt_sql_config_getParameter_str (CONFIG_GROUP, "log.type", "");
    if (tmp_param[0] == '\0')
    {
	Application.settings.log_type = FNT_LOG_T_NONE;
    }
    else
    {
	Application.settings.log_type =
	    fnt_log_getLogTypeByName ((const char *) tmp_param);
    }
    g_free (tmp_param);

    Application.settings.log_level =
	fnt_sql_config_getParameter_int (CONFIG_GROUP, "node.address",
					 LOG_critical);

    switch (Application.settings.log_type)
    {
    case FNT_LOG_T_SYSLOG:
	Application.settings.facility_or_fname =
	    fnt_sql_config_getParameter_str (CONFIG_GROUP, "log.facility",
					     "local7");
	break;
    case FNT_LOG_T_FILE:
	Application.settings.facility_or_fname =
	    fnt_sql_config_getParameter_str (CONFIG_GROUP, "log.file",
					     CONFIG_GROUP ".log");
	break;
    default:
	Application.settings.facility_or_fname = NULL;
	break;
    }


    Application.settings.pid_file =
	fnt_sql_config_getParameter_str (CONFIG_GROUP, "pid.file",
					 CONFIG_GROUP ".pid");

    Application.settings.io_check_interval =
	fnt_sql_config_getParameter_str (CONFIG_GROUP, "io.check_interval",
					 0);

    Application.settings.listen_ip =
	fnt_sql_config_getParameter_str (CONFIG_GROUP, "listen.ip",
					 "0.0.0.0");

    Application.settings.listen_port =
	fnt_sql_config_getParameter_int (CONFIG_GROUP, "listen.port", 24441);


    Application.settings.debug_stream =
	fnt_sql_config_getParameter_bool (CONFIG_GROUP, "debug.stream",
					  false);

    Application.settings.debug_channel =
	fnt_sql_config_getParameter_bool (CONFIG_GROUP, "debug.channel",
					  false);

    Application.settings.ssl_private_key =
	fnt_sql_config_getParameter_str (CONFIG_GROUP, "ssl.key.private",
					  "");
    Application.settings.ssl_public_key =
	fnt_sql_config_getParameter_str (CONFIG_GROUP, "ssl.key.public", "");

    Application.settings.isSettingsInit = true;
    return true;
}
